//
//  CompanyMenuTableViewController.m
//  CoreDataLibraryApp
//
//  Created by Duncan Groenewald on 13/02/2014.
//  Copyright (c) 2014 OSSH Pty Ltd. All rights reserved.
//

#import "CompanyMenuTableViewController.h"
#import "PeopleViewController.h"
#import "CompanyDetailViewController.h"

#define MENU_COMPANY_CONTACT    @"Companies & Contacts"
#define MENU_PROJECTS           @"Projects"
#define MENU_FINANCE            @"Finance"
#define MENU_QUALITY            @"Quality"
#define MENU_ADMIN              @"Administration"

#define SUBMENU_COMPANY          @"Details"
#define SUBMENU_PEOPLE           @"Employees"
#define SUBMENU_CONTACTS         @"Contacts"
#define SUBMENU_TASKS            @"Tasks"


@interface CompanyMenuTableViewController ()

@end

@implementation CompanyMenuTableViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    //FLOG(@" called");
    
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
        
    }
    return self;
}

- (void)viewDidLoad
{
    //FLOG(@" called");
    [super viewDidLoad];
    [self loadMenuData];
    _isLoaded = YES;
    
    // Title of menu
    //self.navigationItem.title = [self.detailItem valueForKey:@"name"];
    self.navigationItem.title = @"Company Details";
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(storeChanged) name:OSStoreChangeNotification
                                               object:nil];
    
    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
    
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
    
    DetailViewManager *detailViewManager = (DetailViewManager*)self.splitViewController.delegate;
    _openingViewController = (OpeningViewController*)detailViewManager.detailViewController;
}
- (void)viewWillAppear:(BOOL)animated  {
    
    FLOG(@" called");
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
        // Set to the opening view controller when we are shown
        DetailViewManager *detailViewManager = (DetailViewManager*)self.splitViewController.delegate;
        
        [detailViewManager showOpeningViewController];
        
    }
}
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


- (void)loadMenuData {
    //FLOG(@" called");
    
    sectionMenus = [[NSMutableDictionary alloc] init];
    sectionsArray = [[NSArray alloc] initWithObjects:MENU_COMPANY_CONTACT, MENU_PROJECTS, MENU_FINANCE, MENU_QUALITY, MENU_ADMIN, nil];
    
    [sectionMenus setObject:[[NSArray alloc] initWithObjects:SUBMENU_COMPANY, SUBMENU_PEOPLE, SUBMENU_CONTACTS, SUBMENU_TASKS, nil] forKey:MENU_COMPANY_CONTACT];
    [sectionMenus setObject:[[NSArray alloc] initWithObjects:@"Invoices", @"Payments", nil] forKey:MENU_FINANCE];
    [sectionMenus setObject:[[NSArray alloc] initWithObjects:@"Issues", @"Risks", nil] forKey:MENU_QUALITY];
    [sectionMenus setObject:[[NSArray alloc] initWithObjects:@"Billing", @"Services", nil] forKey:MENU_ADMIN];
    
}
-(void)storeChanged {
    //FLOG(@"storeChanged called");
    _isLoaded = YES;
    [(UITableView*)self.view reloadData];
}
#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    //FLOG(@" called");
    return [sectionsArray count];
}
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    FLOG(@" called for section %d", section);
    
    if (section == 1) {
        id <NSFetchedResultsSectionInfo> sectionInfo = [self.fetchedResultsController sections][0];
        NSInteger count = [sectionInfo numberOfObjects];
        FLOG(@" section has %d rows", count);
        return count;
    } else {
    
        return [[sectionMenus objectForKey:[sectionsArray objectAtIndex:section]] count];
    }
}
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    //FLOG(@" called");
    if (section == 0) {
        return [self.detailItem valueForKey:@"name"];
    } else
    return [[sectionsArray objectAtIndex:section] description];
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}
- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
    //FLOG(@"  called for %@", [sectionsArray objectAtIndex:[indexPath section]]);
    //static NSString *simpleTableIdentifier = @"SimpleTableItem";
    //static NSString *inactiveTableIdentifier = @"InactiveTableItem";
    
    if ([[sectionsArray objectAtIndex:[indexPath section]] isEqualToString:MENU_PROJECTS]) {
        //FLOG(@"  menu item is ProjectType");
        NSUInteger indexArr[] = {0,indexPath.item};
        NSIndexPath *indexP = [[NSIndexPath alloc] initWithIndexes:indexArr length:2];
        NSManagedObject *object = [self.fetchedResultsController objectAtIndexPath:indexP];
        
        //cell = [tableView dequeueReusableCellWithIdentifier:simpleTableIdentifier];
        
        //if (cell == nil) {
        //    cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:simpleTableIdentifier];
        //}
        
        cell.textLabel.textColor = [UIColor blackColor];
        
        cell.textLabel.text = [object valueForKeyPath:@"name"];
    
        //return cell;
    
    } else {
    
    NSObject *menuItem = [[sectionMenus objectForKey:[sectionsArray objectAtIndex:[indexPath section]]] objectAtIndex:[indexPath row]];
    NSString *menu = [menuItem description];
    
    if ([self isDisabled:menu] || !_isLoaded)
    {
        //cell = [tableView dequeueReusableCellWithIdentifier:inactiveTableIdentifier];
        
        //if (cell == nil) {
        //    cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:inactiveTableIdentifier];
        //}
        
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.textLabel.textColor = [UIColor grayColor];
        cell.backgroundView.backgroundColor = [UIColor lightGrayColor];
        
        if (!_isLoaded) {
            cell.accessoryType = UITableViewCellAccessoryNone;
            UIActivityIndicatorView *progressView = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
            cell.accessoryView = progressView;
            [progressView startAnimating];
        }
        
    }
    else
    {
        //cell = [tableView dequeueReusableCellWithIdentifier:simpleTableIdentifier];
        
        //if (cell == nil) {
        //    cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:simpleTableIdentifier];
        //}
        
        cell.textLabel.textColor = [UIColor blackColor];
        
    }
    
    cell.textLabel.text = menu;
    }
    
    //return cell;
}
- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSString *menu = [[[sectionMenus objectForKey:[sectionsArray objectAtIndex:[indexPath section]]] objectAtIndex:[indexPath row]] description];
    if ([self isDisabled:menu] || !_isLoaded)
        return nil;
    else
        return indexPath;
    
}
- (BOOL)isDisabled:(NSString*)menu {
    
    if (!([menu isEqualToString:SUBMENU_COMPANY] ||
          [menu isEqualToString:SUBMENU_PEOPLE])
        )
        return YES;
    else
        return NO;
    
}
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    //FLOG(@" called");
    NSObject *menu = [[sectionMenus objectForKey:[sectionsArray objectAtIndex:[indexPath section]]] objectAtIndex:[indexPath row]];
    
    NSString *menuString = menu.description;
    
    if ([menuString isEqualToString:SUBMENU_COMPANY]) {
        
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
            
            // Create and configure a new detail view controller appropriate for the selection.
            CompanyDetailViewController *newTableViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"CompanyDetailViewController"];
            newTableViewController.detailItem = self.detailItem;
            
            [self.navigationController pushViewController:newTableViewController animated:YES];
        } else {
            // We want to highlight the selected row
            [self.tableView selectRowAtIndexPath:indexPath animated:YES scrollPosition:UITableViewScrollPositionNone];
            
            FLOG(@" Device is iPad");
            DetailViewManager *detailViewManager = (DetailViewManager*)self.splitViewController.delegate;
            FLOG(@" detailViewManager is %@", detailViewManager);
            
            // Create and configure a new detail view controller appropriate for the selection.
            CompanyDetailViewController *newTableViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"CompanyDetailViewController"];
            newTableViewController.detailItem = self.detailItem;
            
            detailViewManager.detailViewController = newTableViewController;
        }
        return;
    }
    if ([menuString isEqualToString:SUBMENU_PEOPLE]) {
        
        PeopleViewController *newTableViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"PeopleViewController"];
        newTableViewController.detailItem = self.detailItem;
        [self.navigationController pushViewController:newTableViewController animated:YES];
        
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
            DetailViewManager *detailViewManager = (DetailViewManager*)self.splitViewController.delegate;
            FLOG(@" detailViewManager is %@", detailViewManager);
            
            // Create and configure a new detail view controller appropriate for the selection.
            PeopleDetailViewController *newTableViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"PeopleDetailViewController"];
            newTableViewController.detailItem = nil;
            
            detailViewManager.detailViewController = newTableViewController;
            
        }
        return;
    }
    ////FLOG(@" Menu %@ selected.", [menu description]);
    //[self performSegueWithIdentifier:[menu description] sender:tableView];
    
}

#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    //FLOG(@" called");
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
    if ([[segue identifier] isEqualToString:@"Companies"]) {
        //[[segue destinationViewController] setDetailItem:];
    }
    
}

#pragma mark - Fetched results controller

- (NSFetchedResultsController *)fetchedResultsController
{
    FLOG(@"  called");

    if (_fetchedResultsController != nil) {
        return _fetchedResultsController;
    }
    
    if (self.detailItem == nil) return nil;
    self.managedObjectContext =[(NSManagedObject*)self.detailItem managedObjectContext];
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    // Edit the entity name as appropriate.
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"ProjectType" inManagedObjectContext:self.managedObjectContext];
    [fetchRequest setEntity:entity];
    
    // Set the batch size to a suitable number.
    [fetchRequest setFetchBatchSize:20];
    
    // Edit the sort key as appropriate.
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES];
    NSArray *sortDescriptors = @[sortDescriptor];
    
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    // Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:self.managedObjectContext sectionNameKeyPath:nil cacheName:nil];
    aFetchedResultsController.delegate = self;
    self.fetchedResultsController = aFetchedResultsController;
    
	NSError *error = nil;
	if (![self.fetchedResultsController performFetch:&error]) {
        // Replace this implementation with code to handle the error appropriately.
        // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
	    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
	    abort();
	}
    
    return _fetchedResultsController;
}

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
    NSLog(@"controllerWillChangeContent called");
    [self.tableView beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath
{
    UITableView *tableView = self.tableView;
    
    // Reset to second section section
    NSUInteger newIndexArr[] = {1,newIndexPath.item};
    NSIndexPath *newIndexP = [[NSIndexPath alloc] initWithIndexes:newIndexArr length:2];
    NSUInteger indexArr[] = {1,indexPath.item};
    NSIndexPath *indexP = [[NSIndexPath alloc] initWithIndexes:indexArr length:2];

    
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:@[newIndexP] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:@[indexP] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeUpdate:
            [self configureCell:[tableView cellForRowAtIndexPath:indexP] atIndexPath:indexP];
            break;
            
        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:@[indexP] withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:@[newIndexP] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    NSLog(@"controllerDidChangeContent called");
    [self.tableView endUpdates];
}
@end

