//
//  DetailViewController.h
//  Common
//
//  Created by Duncan Groenewald on 12/07/13.
//  Copyright (c) 2013 Duncan Groenewald. All rights reserved.
//

#import <UIKit/UIKit.h>
#import "DetailViewManager.h"
#import "Macros.h"
//#import "OSImageViewController.h"

static const CGFloat KEYBOARD_ANIMATION_DURATION = 0.3;
static const CGFloat MINIMUM_SCROLL_FRACTION = 0.2;
static const CGFloat MAXIMUM_SCROLL_FRACTION = 0.8;
static const CGFloat PORTRAIT_KEYBOARD_HEIGHT = 216;
static const CGFloat LANDSCAPE_KEYBOARD_HEIGHT = 162;

#define mustOverride() @throw [NSException exceptionWithName:NSInvalidArgumentException reason:[NSString stringWithFormat:@"%s must be overridden in a subclass/category", __PRETTY_FUNCTION__] userInfo:nil]
#define methodNotImplemented() mustOverride()

@interface DetailViewController : UIViewController <SubstitutableDetailViewController, UITextFieldDelegate, UITextViewDelegate, UIActionSheetDelegate> {
    UIButton *_activeButton;
    CGFloat animatedDistance;
    UIActionSheet *dateActionSheet;
    float _originalHeight;
    
    IBOutlet UISegmentedControl *sizeButton;
    bool _isCancelled;
    bool _wasEdited;
    
    UIActionSheet *_attachmentMenuSheet;
    NSTextAttachment *_attachment;
}

@property (strong, nonatomic) NSManagedObject *detailItem;
@property (strong, nonatomic) NSManagedObjectContext *managedObjectContext;

@property (nonatomic, retain) IBOutlet UIScrollView *scrollView;
@property (nonatomic, retain) IBOutlet UIView *contentView;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *contentViewWidth;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *contentViewHeight;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *keyboardSpace;

@property (nonatomic, retain) IBOutlet UINavigationBar *navigationBar;

/// SubstitutableDetailViewController
@property (nonatomic, retain) UIBarButtonItem *navigationPaneBarButtonItem;
@property (nonatomic, retain) UIView *activeField;
@property (nonatomic, retain) UIBarButtonItem *cancelButton;

@property (nonatomic, retain) NSString *titleString;

- (void)drawBorder: (UIView*)tv;
- (NSAttributedString *)rtfText: (NSData *) rtfd;
- (void) setViewTitle:(NSString *)title_;
- (NSString*)getViewTitle;
- (void)configureView;
- (void)endEditingForAll;
- (void)registerToHideKeyboard:(id)control;
- (void)showDatePicker:(id)sender;
- (void)refreshUI:(NSNotification*)note;
- (void)displayItem;
- (NSString*)stringFromCurrency:(NSNumber*)number;
- (NSString*)stringFromNumber:(NSNumber*)number;
- (NSNumber*)numberFromString:(NSString*)string;

@end