//
//  DetailViewController.m
//  Common
//
//  Created by Duncan Groenewald on 12/07/13.
//  Copyright (c) 2013 Duncan Groenewald. All rights reserved.
//
//  Subclass this and implement at least the following
//  - (void) save
//  {
//      // Implement code to save data from UI to NSManagedObject (_detailItem)
//  }
//  - (void) setEditable: (BOOL) editable
//  {
//      // Implement code to set displayed fields to editable or not
//  }
//  - (void) configureView
//  {
//      // Must call this first
//      [super configureView];
//
//      // Implement code to populate the UI with data from NSManagedObject (_detailItem)
//  }

#import <Foundation/Foundation.h>
#import "DetailViewController.h"
#import <QuartzCore/QuartzCore.h>
#import "UIPopoverController+iPhone.h"
//#import "OSTextView.h"
//#import "OSTextAttachment.h"
#import "OSCDStackManager.h"


@interface DetailViewController ()
@property (strong, nonatomic) UIPopoverController *masterPopoverController;
- (void)configureView;
@end

@implementation DetailViewController

@synthesize detailItem = _detailItem;
@synthesize titleString = _titleString;
@synthesize managedObjectContext = _managedObjectContext;

@synthesize navigationBar = _navigationBar;
@synthesize masterPopoverController = _masterPopoverController;

#pragma mark - Managing the detail item

- (void)setDetailItem:(NSManagedObject*)newDetailItem
{
    //LOG(@"setDetailItem called");
    //FLOG(@"setDetailItem called for %@", [newDetailItem description]);
    //Set the detail item to nil so save does not happen if in editing mode
    _detailItem = nil;
    //Turn off EDITING mode if its still on but don't save so we call the Super to do this to avoid our method which calls save()
    [super setEditing:NO animated:YES];
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
        self.navigationItem.leftBarButtonItem = nil;
    else
        [self.navigationBar.topItem setLeftBarButtonItem:nil
                                                animated:YES];

    [self setEditable:NO];
    
    if (_detailItem != newDetailItem) {
        _detailItem = newDetailItem;
        
        // Update the view.
        [self configureView];
    }
    
    // Don't dismiss is we are setting the detailed object to nil
    if (_detailItem != nil)
        if (self.masterPopoverController != nil) {
            [self.masterPopoverController dismissPopoverAnimated:YES];
        }
}
- (void)refreshUI:(NSNotification*)note {
    FLOG(@" called");
    [[NSOperationQueue mainQueue] addOperationWithBlock:^ {
        //Your code goes in here
        //FLOG(@" about to call displayItem");
        [self displayItem];
        //FLOG(@" displayItem has been called");
    }];
   
}
- (void)displayItem {
    FLOG(@"displayItem called but you should override this method!");
}
- (void) setViewTitle:(NSString *)title_
{
    
    _titleString = title_;
}
- (NSString*)getViewTitle
{
    return _titleString;
}
// Save any changes to the object
- (void)save {
    mustOverride();
}
- (void)setEditable:(BOOL)editable {
    mustOverride();
}
- (void)registerToHideKeyboard:(id)control {
    [control addTarget:self action:@selector(endEditingForAll) forControlEvents:UIControlEventAllEvents];
    [control setDetailViewController:self];
}
- (void)endEditingForAll {
    [self.view endEditing:YES];
}
- (void)configureView {
    //LOG(@"configureView called");
    self.navigationBar.topItem.title = _titleString;
    //FLOG(@" title is %@", _titleString);
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
        self.navigationItem.rightBarButtonItem = self.editButtonItem;
    else
        [self.navigationBar.topItem setRightBarButtonItem:self.editButtonItem animated:NO];
}
- (void)cancel
{   FLOG(@"cancel called.");
    _isCancelled = YES;
    [self setEditing:NO animated:YES];
    _isCancelled = NO;
    [self displayItem];
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
    [super setEditing:editing animated:animated];
    if (editing) {
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
            //LOG(@" set cancel button for iPhone");
            self.navigationItem.leftBarButtonItem = _cancelButton;
        }
        else {
            //LOG(@" set cancel button for iPad");
            //FLOG(@" navigationBar is %@", self.navigationBar);
            [self.navigationBar.topItem setLeftBarButtonItem:_cancelButton
                                            animated:YES];
        }
    } else {
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
            self.navigationItem.leftBarButtonItem = nil;
        else
        {
            // Put the navigation menu back if in portrait mode or set to nil of not
            if (self.navigationPaneBarButtonItem)
                [self.navigationBar.topItem setLeftBarButtonItem:self.navigationPaneBarButtonItem
                                                        animated:YES];
            else
                [self.navigationBar.topItem setLeftBarButtonItem:nil
                                                        animated:YES];
        }
   }

    //NSLog(@"DetailViewController.setEditing(%d) called.", editing);
    //NSLog(@"DetailViewController.editable = %d", self.editing);

    //set fields to allow/disallow editing
    [self setEditable:editing];

    // Save if not editing any more
    if (!editing)
    {
        if (_detailItem != nil && !_isCancelled) // Dont try saving if its nil
            [self save];
    }
    return;
}
- (void)drawBorder: (UIView*)tv
{
    tv.layer.borderWidth=1.0f;
    tv.layer.borderColor = [[UIColor lightGrayColor] CGColor];
    tv.layer.cornerRadius = 8;
    
}

- (void)viewDidLayoutSubviews
{
    
    FLOG(@"viewDidLayoutSubviews called");
    FLOG(@" detailView.frame.size.width is %f", self.view.frame.size.width);
    FLOG(@" detailView.frame.size.height is %f", self.view.frame.size.height);
    FLOG(@" scrollView.frame.size.width is %f", self.scrollView.frame.size.width);
    FLOG(@" scrollView.frame.size.height is %f", self.scrollView.frame.size.height);
    FLOG(@" contentView.frame.size.width is %f", self.contentView.frame.size.width);
    FLOG(@" contentView.frame.size.height is %f", self.contentView.frame.size.height);
    /**/
    self.contentViewWidth.constant = self.scrollView.frame.size.width;
    if (self.contentViewHeight != nil) {
        //FLOG(@" Setting contentView height to scrollView height");
        self.contentViewHeight.constant = self.scrollView.frame.size.height;
    }
    
    FLOG(@" contentView.frame.size.width is %f", self.contentView.frame.size.width);
    FLOG(@" contentView.frame.size.height is %f", self.contentView.frame.size.height);
    [self.scrollView setContentSize:self.contentView.frame.size];
    
    [self.view layoutSubviews];
    
}
- (void)showSizeOfViews
{
    
    FLOG(@" DetailView size is %fw, %fh", self.view.frame.size.width, self.view.frame.size.height);
    
    FLOG(@" ScrollView size is %fw, %fh", self.scrollView.frame.size.width, self.contentView.frame.size.height);
    
    FLOG(@" ScrollView content size is %fw, %fh", self.scrollView.contentSize.width, self.contentView.frame.size.height);
    
    FLOG(@" contentView size is %fw, %fh", self.contentView.frame.size.width,self.contentView.frame.size.height);
    
}
- (void)viewDidLoad
{
    //LOG(@"viewDidLoadCalled");
    [super viewDidLoad];
    _isCancelled = NO;
    _wasEdited = NO;
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(refreshUI:) name:OSDataUpdatedNotification object:nil];
    
    if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber_iOS_6_1) {
        // Load resources for iOS 6.1 or earlier
    } else {
        // Load resources for iOS 7 or later
        self.edgesForExtendedLayout = UIRectEdgeNone;
    }
    
    _cancelButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(cancel)];

    [self setEditable:NO];
    [self configureView];
    
    /*
    if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber_iOS_6_1) {
        // Load resources for iOS 6.1 or earlier
    } else {
        // Load resources for iOS 7 or later
        self.edgesForExtendedLayout = UIRectEdgeNone;
        self.automaticallyAdjustsScrollViewInsets = NO;
    }
*/
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification
                                               object:self.view.window];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWasShown:)
                                                 name:UIKeyboardDidShowNotification object:nil];
    
    // Do any additional setup after loading the view, typically from a nib.
    // -setNavigationPaneBarButtonItem may have been invoked when before the
    // interface was loaded.  This will occur when setNavigationPaneBarButtonItem
    // is called as part of DetailViewManager preparing this view controller
    // for presentation as this is before the view is unarchived from the NIB.
    // When viewDidLoad is invoked, the interface is loaded and hooked up.
    // Check if we are supposed to be displaying a navigationPaneBarButtonItem
    // and if so, add it to the navigationBar.
    if (self.navigationPaneBarButtonItem)
        [self.navigationBar.topItem setLeftBarButtonItem:self.navigationPaneBarButtonItem
                                                animated:NO];
    /*
    if (dateActionSheet) {
        for (UIView* subview in dateActionSheet.subviews) {
            subview.frame = dateActionSheet.bounds;
        }
    }
     */
    [self setTextDelegates];
    
    //UIMenuItem *menuItem = [[UIMenuItem alloc] initWithTitle:@"View Image 2" action:@selector(viewImage)];
    //[[UIMenuController sharedMenuController] setMenuItems:[NSArray arrayWithObject:menuItem]];

}
/*
- (BOOL)canPerformAction:(SEL)action withSender:(id)sender {
    LOG(@"canPerformAction: called");
    
    if (action == @selector(viewImage)) {
        //if (self.selectedRange.length > 0) {
            
            // Check for an attachment
           // NSTextAttachment *attachment = [[self textStorage] attribute:NSAttachmentAttributeName atIndex:self.selectedRange.location effectiveRange:NULL];
            //if (attachment)
                return YES;
           // else
          //      return NO;
        //}
    }
    return NO;
}
 */
- (void)setTextDelegates
{
    // Iterate through contentsView to find text fields and set ourselves as the delegate
    for (UIView *subview in self.contentView.subviews)
    {
        if ([subview isKindOfClass:[UITextField class]]) {
            [(UITextField*)subview setDelegate:self];
        } else
        if ([subview isKindOfClass:[UITextView class]]) {
            
            [(UITextView*)subview setDelegate:self];
            
        }
        /*
        if ([subview isKindOfClass:[OSTextView class]]) {
            
            // If a Size Button is provided assign all UITextViews as its target
            if (sizeButton)
                [sizeButton addTarget:subview action:@selector(sizeText:) forControlEvents:UIControlEventValueChanged];

        }
         */
    }
}
- (void) viewWillDisappear:(BOOL)animated {
    FLOG(@" called");
    
    if (_wasEdited) {
        [self save];
    }
    
    [super viewWillDisappear:animated];
    // Release any retained subviews of the main view.
    //self.contentViewWidth = nil;
    //self.contentViewHeight = nil;
    //self.contentView = nil;
    //self.scrollView = nil;
    //self.cancelButton=nil;
    
    //[[NSNotificationCenter defaultCenter] removeObserver:self];

    // unregister for keyboard notifications while not visible.
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIKeyboardWillShowNotification
                                                  object:nil];
    // unregister for keyboard notifications while not visible.
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIKeyboardDidShowNotification
                                                  object:nil];
    // unregister for keyboard notifications while not visible.
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIKeyboardWillHideNotification
                                                  object:nil];
    
    
}
- (void)keyboardWillHide:(NSNotification *)n
{
    FLOG(@" called");
    
    // Apples recommended method
    UIEdgeInsets contentInsets = UIEdgeInsetsZero;
    self.scrollView.contentInset = contentInsets;
    self.scrollView.scrollIndicatorInsets = contentInsets;
    /*
    if ([_activeField isKindOfClass:[OSTextView class]]) {
        [(OSTextView*)_activeField restoreHeight];
    }
     */
}
- (void)keyboardWillShow:(NSNotification *)n
{
    FLOG(@" called");
    
    NSDictionary* userInfo = [n userInfo];
    NSTimeInterval animationDuration = [[userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey] doubleValue];
    // get the size of the keyboard
    CGRect keyboardRect = [[userInfo objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue];
    CGRect kr = [self.view convertRect:keyboardRect fromView:nil];
    CGSize keyboardSize = kr.size;
    
    self.keyboardSpace.constant = -keyboardSize.height;
    
    [UIView animateWithDuration:animationDuration animations:^{
        [self.view layoutIfNeeded];
    }];
    
}
// Called when the UIKeyboardDidShowNotification is sent.
- (void)keyboardWasShown:(NSNotification*)aNotification
{
    FLOG(@" called");
    NSDictionary* info = [aNotification userInfo];
    CGRect keyboardRect = [[info objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue];
    CGRect kr = [self.view convertRect:keyboardRect fromView:nil];
    CGSize kbSize = kr.size;
    
    UIEdgeInsets contentInsets = UIEdgeInsetsMake(0.0, 0.0, kbSize.height, 0.0);
    self.scrollView.contentInset = contentInsets;
    self.scrollView.scrollIndicatorInsets = contentInsets;
    
    // If active text field is hidden by keyboard, scroll it so it's visible
    // and resize it so it fits into available space exactly
    // Your application might not need or want this behavior.
    CGRect fRect = _activeField.frame;  //Need to add co-ordinates of scrollView to this
    CGRect aRect = self.scrollView.frame;

    /*
    if ([_activeField isKindOfClass:[TextView class]]) {
        float height = aRect.size.height;
        _originalHeight = fRect.size.height;

        [(TextView*)_activeField adjustHeight:(height-kbSize.height - 35)];
    }
     */

    fRect.origin.y += aRect.origin.y;
    
    aRect.size.height -= kbSize.height;

    
    float maxOffset = self.contentView.frame.size.height - (self.scrollView.frame.size.height - kbSize.height);
    
    if (_activeField !=nil) {
        float m = MIN(_activeField.frame.origin.y-35,maxOffset);
            CGPoint scrollPoint = CGPointMake(0.0, m);
            [self.scrollView setContentOffset:scrollPoint animated:YES];
    }
}

#pragma mark - UITextField Delegate

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    //FLOG(@"textFieldShouldBeginEditing called");
    _activeField = textField;
    return YES;
}
- (void)textFieldDidBeginEditing:(UITextField *)textField
{
    //FLOG(@"textFieldDidBeginEditing called");
    _activeField = textField;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
    //FLOG(@"textFieldDidEndEditing called");
    _activeField = nil;
}

#pragma mark - UITextView Delegate

- (BOOL)textViewShouldBeginEditing:(UITextView *)textView {
    //FLOG(@"textViewShouldBeginEditing called");
    _activeField = textView;
    return YES;
}
- (void)textViewDidBeginEditing:(UITextView *)textView
{
    //FLOG(@"textViewDidBeginEditing called");
    _activeField = textView;
}

- (void)textViewDidEndEditing:(UITextView *)textView
{
    //FLOG(@"textViewDidEndEditing called");
    _activeField = nil;
}

// Called when the UIKeyboardWillHideNotification is sent
- (void)keyboardWillBeHidden:(NSNotification*)aNotification
{
    //LOG(@"Keyboard will be hidden");
    
    UIEdgeInsets contentInsets = UIEdgeInsetsZero;
    self.scrollView.contentInset = contentInsets;
    self.scrollView.scrollIndicatorInsets = contentInsets;
}
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    //LOG(@"initWithNibName called");
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.navigationBar.topItem.title = NSLocalizedString(@"", @"");
    }
    return self;
}

// -------------------------------------------------------------------------------
//	setNavigationPaneBarButtonItem:
//  Custom implementation for the navigationPaneBarButtonItem setter.
//  In addition to updating the _navigationPaneBarButtonItem ivar, it
//  reconfigures the toolbar to either show or hide the
//  navigationPaneBarButtonItem.
// -------------------------------------------------------------------------------
- (void)setNavigationPaneBarButtonItem:(UIBarButtonItem *)navigationPaneBarButtonItem
{
    //LOG(@"setNavigationPaneBarButtonItem called");
    if (navigationPaneBarButtonItem != _navigationPaneBarButtonItem) {
        // Add the popover button to the left navigation item.
        [self.navigationBar.topItem setLeftBarButtonItem:navigationPaneBarButtonItem
                                                animated:NO];
        
        // [_navigationPaneBarButtonItem release];
        _navigationPaneBarButtonItem = navigationPaneBarButtonItem;
    }
}


/*  This function transforms the Attributed text (RTFD) to plain text and
 *  also strips out some characters to prevent XML errors
 *
 */
- (NSAttributedString *)rtfText: (NSData *) rtfd;
{
	NSString *name = NSKeyedUnarchiveFromDataTransformerName;
    NSAttributedString *rtfData;
	if (rtfd == nil) {
		
		return nil;
	}
    
    @try {
        
        rtfData = (NSAttributedString *)[[NSValueTransformer valueTransformerForName: name]
                                         transformedValue:rtfd];
        
	} @catch (NSException *e) {
        NSLog(@"exception: %@", e);
        return (NSAttributedString *)@"Error converting to text.";
    }
	return rtfData;
}
- (void)scrollToVisible2
{
    FLOG(@" called");
    FLOG(@" active button is %@", _activeButton);
    CGRect textFieldRect =
    [self.view.window convertRect:_activeButton.bounds fromView:_activeButton];
    CGRect viewRect =
    [self.view.window convertRect:self.view.bounds fromView:self.view];
    CGFloat midline = textFieldRect.origin.y + 0.5 * textFieldRect.size.height;
    CGFloat numerator =
    midline - viewRect.origin.y
    - MINIMUM_SCROLL_FRACTION * viewRect.size.height;
    CGFloat denominator =
    (MAXIMUM_SCROLL_FRACTION - MINIMUM_SCROLL_FRACTION)
    * viewRect.size.height;
    CGFloat heightFraction = numerator / denominator;
    if (heightFraction < 0.0)
    {
        heightFraction = 0.0;
    }
    else if (heightFraction > 1.0)
    {
        heightFraction = 1.0;
    }
    UIInterfaceOrientation orientation =
    [[UIApplication sharedApplication] statusBarOrientation];
    if (orientation == UIInterfaceOrientationPortrait ||
        orientation == UIInterfaceOrientationPortraitUpsideDown)
    {
        animatedDistance = floor(PORTRAIT_KEYBOARD_HEIGHT * heightFraction);
    }
    else
    {
        animatedDistance = floor(LANDSCAPE_KEYBOARD_HEIGHT * heightFraction);
    }
    CGRect viewFrame = self.view.frame;
    viewFrame.origin.y -= animatedDistance;
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationBeginsFromCurrentState:YES];
    [UIView setAnimationDuration:KEYBOARD_ANIMATION_DURATION];
    
    [self.view setFrame:viewFrame];
    
    [UIView commitAnimations];
}
- (void)scrollToVisible:(float) height
{
    //LOG(@"scrollToVisible: called");
    //FLOG(@" active button is %@", _activeButton);
    CGRect textFieldRect = [self.view.window convertRect:_activeButton.bounds fromView:_activeButton];
    CGRect viewRect = [self.view.window convertRect:self.view.bounds fromView:self.view];
    CGFloat midline = textFieldRect.origin.y + 0.5 * textFieldRect.size.height;
    CGFloat numerator = midline - viewRect.origin.y - MINIMUM_SCROLL_FRACTION * viewRect.size.height;
    CGFloat denominator = (MAXIMUM_SCROLL_FRACTION - MINIMUM_SCROLL_FRACTION) * viewRect.size.height;
    CGFloat heightFraction = numerator / denominator;
    if (heightFraction < 0.0)
    {
        heightFraction = 0.0;
    }
    else if (heightFraction > 1.0)
    {
        heightFraction = 1.0;
    }
    UIInterfaceOrientation orientation =
    [[UIApplication sharedApplication] statusBarOrientation];
    if (orientation == UIInterfaceOrientationPortrait ||
        orientation == UIInterfaceOrientationPortraitUpsideDown)
    {
        animatedDistance = floor(PORTRAIT_KEYBOARD_HEIGHT * heightFraction);
    }
    else
    {
        animatedDistance = floor(LANDSCAPE_KEYBOARD_HEIGHT * heightFraction);
    }
    animatedDistance = floor(height * heightFraction);
    CGRect viewFrame = self.view.frame;
    viewFrame.origin.y -= animatedDistance;
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationBeginsFromCurrentState:YES];
    [UIView setAnimationDuration:KEYBOARD_ANIMATION_DURATION];
    
    [self.view setFrame:viewFrame];
    
    [UIView commitAnimations];
}
- (void)scrollBack
{
    CGRect viewFrame = self.view.frame;
    viewFrame.origin.y += animatedDistance;
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationBeginsFromCurrentState:YES];
    [UIView setAnimationDuration:KEYBOARD_ANIMATION_DURATION];
    
    [self.view setFrame:viewFrame];
    
    [UIView commitAnimations];
}
- (void)showDatePicker:(id)sender {
    //LOG(@"showDatePicker called");
    return;
    float toolbarHeight, datePickerHeight, actionSheetHeight;
    
    _activeButton=sender;
    [self endEditingForAll];
    //[self scrollToVisible2];
    
    dateActionSheet = [[UIActionSheet alloc]initWithTitle:@"Date" delegate:self cancelButtonTitle:nil destructiveButtonTitle:nil otherButtonTitles:nil  , nil];
    dateActionSheet.actionSheetStyle = UIActionSheetStyleDefault;
    dateActionSheet.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    
    UIDatePicker *datePicker = [[UIDatePicker alloc] initWithFrame:CGRectZero];
    datePicker.datePickerMode = UIDatePickerModeDate;
    datePicker.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    [datePicker addTarget:_activeButton action:@selector(datePickerValueChanged:) forControlEvents:UIControlEventValueChanged ];
    NSDateFormatter *df = [[NSDateFormatter alloc] init];
    df.dateStyle = NSDateFormatterMediumStyle;
    CGRect frame = datePicker.frame;
    [self showDimensions:@"datePicker" rect:frame];
    frame.origin.x = 0.0;
    frame.origin.y = 40.0;
    datePickerHeight = frame.size.height;
    datePicker.frame = frame;
    [dateActionSheet addSubview:datePicker];
    //[dateActionSheet addSubview:self.datePickerView];
    
    UIToolbar *tools=[[UIToolbar alloc]initWithFrame:CGRectMake(0, 0,320,40)];
    tools.barStyle=UIBarStyleBlackOpaque;
    tools.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    [dateActionSheet addSubview:tools];
    toolbarHeight = tools.frame.size.height;
    [self showDimensions:@"tools" rect:tools.frame];
    
    actionSheetHeight = toolbarHeight+datePickerHeight;
    
    UIBarButtonItem *doneButton=[[UIBarButtonItem alloc]initWithTitle:@"Done" style:UIBarButtonItemStyleBordered target:self action:@selector(btnActinDoneClicked)];
    doneButton.imageInsets=UIEdgeInsetsMake(200, 6, 50, 25);    
    
    UIBarButtonItem *flexSpace= [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    
    NSArray *array = [[NSArray alloc]initWithObjects:flexSpace,flexSpace,doneButton,nil];
    
    [tools setItems:array];
    
    
    //picker title
    UILabel *lblPickerTitle=[[UILabel alloc]initWithFrame:CGRectMake(60,8, 200, 25)];
    lblPickerTitle.text=@"Select Date";
    lblPickerTitle.backgroundColor=[UIColor clearColor];
    lblPickerTitle.textColor=[UIColor whiteColor];
    lblPickerTitle.textAlignment = NSTextAlignmentCenter;
    lblPickerTitle.font=[UIFont boldSystemFontOfSize:15];
    [tools addSubview:lblPickerTitle];
    
    
    //cell.detailTextLabel.text = [self datePickerValueChangedd:datePicker];
    
    [self scrollToVisible:actionSheetHeight/2];
    
    //[dateActionSheet showFromRect:CGRectMake(0, 0, 320, 0) inView:self.view animated:YES];
    [dateActionSheet showInView:self.view];
    [dateActionSheet setBounds:CGRectMake(0,(toolbarHeight+datePickerHeight)/2-20, 320, toolbarHeight+datePickerHeight)];
    //[self showDimensions:@"dateActionSheet" rect:dateActionSheet.frame];
}
- (void)showDimensions:(NSString*)name rect:(CGRect)frame {
    FLOG(@"%@ dimensions are:", name);
    FLOG(@"  frame.origin.x = %f", frame.origin.x);
    FLOG(@"  frame.origin.y = %f", frame.origin.y);
    FLOG(@"  frame.size.width = %f", frame.size.width);
    FLOG(@"  frame.size.height = %f", frame.size.height);
    
}

-(void)btnActinDoneClicked {
    //LOG(@"btnActinDoneClicked called");
    [dateActionSheet dismissWithClickedButtonIndex:1 animated:YES];
    [dateActionSheet removeFromSuperview];
    [self scrollBack];
    
}
- (BOOL)textView:(UITextView *)textView shouldInteractWithTextAttachment:(NSTextAttachment *)textAttachment inRange:(NSRange)characterRange {
    
    // save in ivar so we can access once action sheet option is selected
    _attachment = textAttachment;
    
    [self attachmentActionSheet:(UITextView *)textView range:characterRange];
    
    return NO;
}
- (void)attachmentActionSheet:(UITextView *)textView range:(NSRange)range {
    
    // get the rect for the selected attachment (if its a big image with top not visible the action sheet
    // will be positioned above the top limit of the UITextView
    // Need to add code to adjust for this.
    CGRect attachmentRect = [self frameOfTextRange:range inTextView:textView];
    FLOG(@" attachmentRect is %fx, %fy, %fw, %fh", attachmentRect.origin.x, attachmentRect.origin.y, attachmentRect.size.width, attachmentRect.size.height);
      _attachmentMenuSheet = [[UIActionSheet alloc] initWithTitle:nil
                                                           delegate:self
                                                  cancelButtonTitle:@"Cancel"
                                             destructiveButtonTitle:nil
                                                  otherButtonTitles:@"Copy Image", @"Save to Camera Roll", @"Open in Viewer", nil];
    
    // Show the sheet
    [_attachmentMenuSheet showFromRect:attachmentRect inView:textView animated:YES];
}
- (CGRect)frameOfTextRange:(NSRange)range inTextView:(UITextView *)textView {
    
    CGRect rect = [textView.layoutManager boundingRectForGlyphRange:range inTextContainer:textView.textContainer];
    
    // Now convert to textView coordinates
    CGRect rectRange = [textView convertRect:rect fromView:textView.textInputView];

    // Now convert to contentView coordinates
    CGRect rectRangeSuper = [self.contentView convertRect:rectRange fromView:textView];
    
    // Get the textView frame
    CGRect rectView = textView.frame;

    // Find the intersection of the two (in the same coordinate space)
    CGRect rectIntersect = CGRectIntersection(rectRangeSuper, rectView);

    // If no intersection then that's weird !!
    if (CGRectIsNull(rectIntersect)) {
        return rectRange;
    }
    
    // Now convert the intersection rect back to textView coordinates
    CGRect rectRangeInt = [textView convertRect:rectIntersect fromView:self.contentView];
    
    return rectRangeInt;
}
- (void)printRect:(CGRect)rect name:(NSString*)name {
    FLOG(@" %@ is %fx, %fy, %fw, %fh", name, rect.origin.x, rect.origin.y, rect.size.width, rect.size.height);
}
- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex {
    if (actionSheet == _attachmentMenuSheet) {
        //FLOG(@"Button %d", buttonIndex);
        switch (buttonIndex) {
                
            case 0:
                //FLOG(@" Copy Image");
                [self copyImageToPasteBoard:[_attachment image]];
                break;
                
            case 1:
                //FLOG(@"  Save to Camera Roll");
                [self saveToCameraRoll:[_attachment image]];
                break;
                
            case 2:
                //FLOG(@"  Open in Viewer");
                [self browseImage:[_attachment image]];
                break;
                
            default:
                break;
        }
    }
}
- (void)saveToCameraRoll:(UIImage*)image {
    UIImageWriteToSavedPhotosAlbum(image, nil, nil, nil);
}
- (void)copyImageToPasteBoard:(UIImage*)image {
    UIPasteboard *pasteboard = [UIPasteboard generalPasteboard];
    NSData *data = UIImagePNGRepresentation(image);
    [pasteboard setData:data forPasteboardType:@"public.png"];
}

-(void)browseImage:(UIImage*)image
{
    //LOG(@"browseImage: called");
    return;
    /*
    OSImageViewController *_imageViewerController = [[OSImageViewController alloc] init];
    UIImage *img = [[UIImage alloc] initWithData:UIImagePNGRepresentation(image)];
    
    //_imageViewerController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
    _imageViewerController.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
    //_imageViewerController.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
    _imageViewerController.modalPresentationStyle = UIModalPresentationFullScreen;
    _imageViewerController.delegate = self;
    //[_imageViewerController setImage:img];
    [self presentViewController:_imageViewerController animated:YES completion:^(void){
        [_imageViewerController setImage:img];
        
    }];
 */
}
- (void)dismissImageViewer {
    [self dismissViewControllerAnimated:YES completion:nil];
}
- (NSString*)stringFromCurrency:(NSNumber*)number {
    NSNumberFormatter *formatter = [[NSNumberFormatter alloc] init];
    [formatter setNumberStyle:NSNumberFormatterCurrencyStyle];
    return [formatter stringFromNumber:number];
}
- (NSString*)stringFromNumber:(NSNumber*)number {
    NSNumberFormatter *formatter = [[NSNumberFormatter alloc] init];
    [formatter setNumberStyle:NSNumberFormatterDecimalStyle];
    return [formatter stringFromNumber:number];
}
- (NSNumber*)numberFromString:(NSString*)string {
    return [NSNumber numberWithFloat:[[string stringByReplacingOccurrencesOfString:@"," withString:@""] floatValue]];
}
// This method is used to replace any custom attachments and MUST be called to convert attributed strings prior to
// saving them in Core Data.
//
// Use it in the detail views Save method.
//
- (NSAttributedString*)replaceCustomAttachments:(NSAttributedString*)attrString {
    
    NSRange range = NSMakeRange(0, attrString.length);
    __block NSMutableDictionary *dict;
    NSMutableAttributedString *mutAttrString = [[NSMutableAttributedString alloc] initWithAttributedString:attrString];
    
    [mutAttrString enumerateAttributesInRange:range options:NSAttributedStringEnumerationLongestEffectiveRangeNotRequired usingBlock:
     ^(NSDictionary *attributes, NSRange range, BOOL *stop) {
         
         dict = [[NSMutableDictionary alloc] initWithDictionary:attributes];
         FLOG(@" range is %d, %d", range.location, range.length);
         
         // Iterate over each attribute and look for attachments
         [dict enumerateKeysAndObjectsUsingBlock:^(id key, id obj, BOOL *stop) {
             
             if ([[key description] isEqualToString:NSAttachmentAttributeName]) {
                 
                 FLOG(@" textAttachment found");
                 FLOG(@" textAttachment class is %@", [obj class]);
                 FLOG(@" textAttachment fileType is %@", [obj fileType]);
                 FLOG(@" textAttachment image is %@", [obj image]);
                 /*
                 if ([obj isKindOfClass:[OSTextAttachment class]]) {
                     FLOG(@"    so replace it");
                     OSTextAttachment *attachment = obj;
                     NSTextAttachment *nsAttachment;
                     
                     if (attachment.image != nil) {
                         FLOG(@" attachment.image found");
                         nsAttachment = [[NSTextAttachment alloc] initWithData:UIImagePNGRepresentation(attachment.image) ofType:@"public.png"];
                     }
                     else {
                         FLOG(@" attachment.image is nil");
                         nsAttachment = [[NSTextAttachment alloc] initWithData:attachment.fileWrapper.regularFileContents ofType:attachment.fileType];
                     }
                     
                     [dict setValue:nsAttachment forKey:key];
                 }
                  */
             }
             
         }];
         
         [mutAttrString setAttributes:dict range:range];
         
     }];
    
    return mutAttrString;
}

@end